/*
 * $Id: PP4hdGradBg.java,v 1.3 2000/05/04 07:17:38 gunterma Exp $
 * (History at end)
 */

/*
 * This file is part of the PPower4 software
 * Copyright (c) 1999,2000  Klaus Guntermann/TU Darmstadt, FB 20, FG SP
 *
 * See the file COPYING in the top level directory for details.
 */

// This implements all the linear gradient background commands.

package de.tu_darmstadt.sp.pp4;

import java.text.*;
import java.util.*;
import de.tu_darmstadt.sp.pp4.*;

public class PP4hdGradBg extends PP4Handler {


  // The comment which we handle.
  public String pattern()
    {
      return("%gradbackground");
    }

  /**
   * Implement the action for the "gradbackground" command, which makes a
   * gradient like background in for our current page.
   * <p>
   * The color descritption, which is decoded here, has the following
   * format:<br>
   * - a letter 'h' or 'v' to indicate the direction.<br>
   * - a letter 't' or 'o' to switch between two or one color supplied.
   *   If there is only one color supplied, we use it as a starting point
   *   to make the background lighter (but not yet white).
   * <p>
   * We do not check the color description contents, if there is any.
   * @param command is the triggered command, sent here for completeness.
   * @param arguments contains a string, which is supposed to encode the
   *  needed color descriptions.
   */
  public void doit(String command, String arguments) throws PP4Exception
    {
      String firstcolor,secondcolor=null;
      arguments=arguments.trim();
      if (arguments==null || arguments.length()<2)
	throw new PP4Exception(command+": missing arguments");
      // check for orientation of gradient first.
      boolean vertical=(arguments.charAt(0)=='v');
      // check for one or two colors given.
      boolean twocolors=(arguments.charAt(1)=='t');
      if (twocolors) {
	// split the argument to get both colors separated
	int splitpos=arguments.indexOf('/');
	if (splitpos==-1)
	  throw new PP4Exception(command+": expect two arguments, got "+
				 arguments);
	firstcolor=arguments.substring(2,splitpos).trim();
	secondcolor=arguments.substring(splitpos+1).trim();
      } else {
	firstcolor=arguments.substring(2).trim();
	// secondcolor=null;
      }

      float [] bounds=getPagesRef().getPageBounds();
      // check, whether we have got our hash initialized.
      if (resulthash==null) resulthash=new Hashtable();
      // try to find a suitable background created earlier to reuse it.
      String hashkey=command+" "+arguments+" "+
	bounds[0]+"/"+bounds[1]+"/"+bounds[2]+"/"+bounds[3];
      Object resultid=resulthash.get(hashkey);
      if (resultid==null) {
	// must create a new background for this combination

	StringBuffer workarea = new StringBuffer(8192);

	String colormodel="", token;
	float deltax=(float)0.0, deltay=(float)0.0, xpos, ypos;
	float [] c1 = new float[4],c2=new float[4],cd=new float[4];
	int colorloop=0,i=0;
	StringTokenizer st=new StringTokenizer(firstcolor," \t\n\r\f");
	for (i=0;i<5;i++) {
	  token=st.nextToken();
	  try{
	    float val=Float.valueOf(token).floatValue();
	    if (i<4) c1[i]=val;
	  } catch (NumberFormatException nfe) {
	    if (i==1 && (token.equals("g")||token.equals("G"))) {
	      colormodel="g"; colorloop=0; break;
	    } else if (i==3 && (token.equals("rg")||token.equals("RG"))) {
	      colormodel="rg"; colorloop=2; break;
	    } else if (i==4 && (token.equals("k")||token.equals("K"))) {
	      colormodel="k"; colorloop=3; break;
	    } else {
	      throw new PP4Exception(command+": bad color spec "+firstcolor);
	    }
	  }
	}
	if (secondcolor==null) {
	  if (colormodel.equals("g"))
	     c2[0]=(float)(c1[0]+(1.0-c1[0])/2.0);
	  else
	  if (colormodel.equals("rg"))
	    for (i=0;i<=colorloop;i++) c2[i]=(float)(c1[i]+(1.0-c1[i])/2.0);
	  else
	    for (i=0;i<=colorloop;i++) c2[i]=(float)(c1[i]-(c1[i]/2.0));
	} else {
	  st=new StringTokenizer(secondcolor," \t\n\r\f"); i=0;
	  for (i=0;i<=colorloop;i++) {
	    token=st.nextToken();
	    try{
	      float val=Float.valueOf(token).floatValue();
	      if (i<4) c2[i]=val;
	    } catch (NumberFormatException nfe) {
	      throw new PP4Exception(command+": color models do not match: "
				     +firstcolor+" vs. "+secondcolor);
	    }
	  }
	  token=st.nextToken().toLowerCase();
	  if (!token.equals(colormodel)) {
	    throw new PP4Exception(command+": color models do not match: "
				   +firstcolor+" vs. "+secondcolor
				   +"/"+token+"/"+colormodel);
	  }
	}
	// because we build the horizontal stripes from right to left
	// (larger values to smaller ones, like from top to down, we have
	// to exchange the color values in this case.
	if (!vertical) {float [] tmpcolor=c2; c2=c1; c1=tmpcolor;}
	// determine the number of different color steps (max 100)
	int steps=100;
	for (i=0;i<=colorloop;i++) {
	  int tmp=(int)((c2[i]-c1[i])*100);
	  if (tmp<0) tmp=-tmp;
	  if (tmp>steps) steps=tmp;
	}
	if (steps>100) steps=100;
	// calculate the delta for each color
	float floatsteps=steps;
	for (i=0;i<=colorloop;i++) {
	  cd[i]=(c2[i]-c1[i])/floatsteps;
	}
	// setup the initial values for the loop and the increments
	if (vertical) {
	  xpos=bounds[0];
	  ypos=bounds[3];
	  deltay=(bounds[3]-bounds[1])/floatsteps;
	  bounds[3]=-deltay;
	} else {
	  ypos=bounds[1];
	  xpos=bounds[2];
	  deltax=(bounds[2]-bounds[0])/floatsteps;
	  bounds[2]=-deltax;
	}

	// we must make sure that the decimals are written in US
	// format, because otherwise Acroread will complain.
	DecimalFormatSymbols usdecforms=
	     new DecimalFormatSymbols(Locale.US);
	DecimalFormat positionformat=new DecimalFormat("0.0##", usdecforms),
	  colorformat=new DecimalFormat("0.0##", usdecforms);
	workarea.append(" q 1 i\n");
	for (int j=0;j<steps-1;j++) {
	  for (i=0;i<=colorloop;i++) {
	    workarea.append(" "+colorformat.format(c1[i]));
	    c1[i]+=cd[i];
	  }
	  workarea.append(" "+colormodel+" "+
			  positionformat.format(xpos)+" "+
			  positionformat.format(ypos)+" "+
			  positionformat.format(bounds[2])+" "+
			  positionformat.format(bounds[3])+" re f\n");
	  ypos-=deltay; xpos-=deltax;
	}
	for (i=0;i<=colorloop;i++)
	  workarea.append(" "+colorformat.format(c1[i]));
	workarea.append(" "+colormodel+" "+
			positionformat.format(xpos)+" "+
			positionformat.format(ypos)+" "+
			positionformat.format(bounds[2])+" "+
			positionformat.format(bounds[3])+" re f\n");
	for (i=0;i<=colorloop;i++)
	  workarea.append(" "+colorformat.format(c1[i]));
	if (vertical)
	  workarea.append(" "+colormodel+" "+
			  positionformat.format(xpos)+" "+
			  positionformat.format(ypos)+" "+
			  positionformat.format(bounds[2])+" "+
			  positionformat.format(deltay)+" re f Q\n");
	else
	  workarea.append(" "+colormodel+" "+
			  positionformat.format(xpos)+" "+
			  positionformat.format(ypos)+" "+
			  positionformat.format(deltax)+" "+
			  positionformat.format(bounds[3])+" re f Q\n");

	// create a new stream to be added to the page.
	resultid=getPagesRef().makeStream(workarea.toString());
	// make it retrievable in the hash
	resulthash.put(hashkey, resultid);
      }
      // Add the newly created or retrieved stream to the page. (This
      // may be delayed, if we already had something written from this
      // page.)
      getPagesRef().addBgStream(resultid);
    }

}

/*
 * $Log: PP4hdGradBg.java,v $
 * Revision 1.3  2000/05/04 07:17:38  gunterma
 * fix background color processing for grayscale values
 *
 * Revision 1.2  1999/08/26 08:02:22  gunterma
 * Make sure that calculations for the background colors are not written
 * in localized conventions.
 *
 * Revision 1.1  1999/08/09 11:56:49  gunterma
 * initial checkin
 *
 */
