/*
 * $Id: PP4hdMonoBg.java,v 1.1 1999/08/09 11:56:49 gunterma Exp $
 * (History at end)
 */

/*
 * This file is part of the PPower4 software
 * Copyright (c) 1999  Klaus Guntermann/TU Darmstadt, FB 20, FG SP
 *
 * See the file COPYING in the top level directory for details.
 */

// This implements the monochrome background creation command.

package de.tu_darmstadt.sp.pp4;

import java.util.*;
import de.tu_darmstadt.sp.pp4.*;

public class PP4hdMonoBg extends PP4Handler {

  // The comment which we handle.
  public String pattern()
    {
      return("%mbackground");
    }

  /**
   * Implement the action for the "mbackground" command, which makes a
   * monochrome background in for our current page.
   * <p>
   * We do not check the color description contents, if there is any.
   * @param command is the triggered command, sent here for completeness.
   * @param arguments contains a string, which is supposed to encode a
   *  suitable color description.
   */
  public void doit(String command, String arguments) throws PP4Exception
    {
      if (arguments==null || arguments.equals(""))
	throw new PP4Exception(command+": missing arguments");
      float [] bounds=getPagesRef().getPageBounds();
      // check, whether we have got our hash initialized.
      if (resulthash==null) resulthash=new Hashtable();
      // try to find a suitable background created earlier to reuse it.
      Object resultid=resulthash.get(command+" "+arguments+" "+bounds);
      if (resultid==null) {// none found.
	StringBuffer workarea=new StringBuffer(512);
	// we must prepare a new background description. First get the
	// page bounds to delimit the colored area properly.
	workarea.append("q ");
	workarea.append("W ");
	workarea.append(bounds[0]);
	workarea.append(" ");
	workarea.append(bounds[1]);
	workarea.append(" m ");
	workarea.append(bounds[0]);
	workarea.append(" ");
	workarea.append(bounds[3]);
	workarea.append(" l ");
	workarea.append(bounds[2]);
	workarea.append(" ");
	workarea.append(bounds[3]);
	workarea.append(" l ");
	workarea.append(bounds[2]);
	workarea.append(" ");
	workarea.append(bounds[1]);
	workarea.append(" l h n ");
	workarea.append(arguments); // insert the color argument here
	// pdflatex will format the color properly.
	workarea.append(" 1 i ");
	workarea.append(bounds[0]);
	workarea.append(" ");
	workarea.append(bounds[1]);
	workarea.append(" ");
	workarea.append(bounds[2]);
	workarea.append(" ");
	workarea.append(bounds[3]);
	workarea.append(" re f Q ");
	// create a new stream to be added to the page.
	resultid=getPagesRef().makeStream(workarea.toString());
	// make it retrievable in the hash
	resulthash.put(command+" "+arguments+" "+bounds, resultid);
      }
      // Add the newly created or retrieved stream to the page. (This
      // may be delayed, if we alreadyhad something written from this
      // page.)
      getPagesRef().addBgStream(resultid);
    }
}

/*
 * $Log: PP4hdMonoBg.java,v $
 * Revision 1.1  1999/08/09 11:56:49  gunterma
 * initial checkin
 *
 */
