/*
 * $Id: PPower4.java,v 1.11 2000/05/04 07:17:39 gunterma Exp $
 * (History at end)
 */

/*
 * This file is part of the PPower4 software
 * Copyright (c) 1999,2000  Klaus Guntermann/TU Darmstadt, FB 20, FG SP
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * Contact the author through the home page of PPower4 at
 *     http://www-sp.iti.informatik.tu-darmstadt.de
 *
 */

// This is the main routine PPower4.java of the
//        Pdf Presentation Post Processor
//
package de.tu_darmstadt.sp.pp4;

import java.io.*;
import java.util.*;
import java.text.*;
import gnu.getopt.*;
import de.tu_darmstadt.sp.pp4.*;

public class PPower4 {

  // Program name for messages.
  private static String progname = "PPower4";

  private static String version= new String("0.8.4");

  // These keep the values to be modified from flags and options.
  private static int verboseLevel = 1;
  private static int compress=1;

  private static void msgout(String msg)
    {
      System.out.println(msg);
    }

  private static void usage()
    {
      msgout(progname+" [-v|-n|-h|-V|--verbose=<n>|--nocompress|--help|--version]"+
	     " infile outfile");
    }

  /**
   * Show program version info according to GPL.
   */
  private static void showVersion()
  {
     System.out.println(
     progname + " version " + version + "\n" +
     "Copyright (C) 1999, 2000 Klaus Guntermann, FG SP, TU Darmstadt\n" +
     progname + " is free software; see the source for copying conditions.\n" +
     "There is NO warranty; not even for MERCHANTABILITY or FITNESS\n" +
     "FOR A PARTICULAR PURPOSE."
    );
    System.exit(0);
  }
  

  // The main routine will do it all.
  public static void main(String args[])
  {

    // Set up command line arguments and parse them.

    int c;
    String optval; // the option argument's value (if any)
    StringBuffer argsb=new StringBuffer();
    LongOpt [] longopts = new LongOpt[4];

    longopts[0] = new LongOpt("help",LongOpt.NO_ARGUMENT, null, 'h');
    longopts[1] = new LongOpt("verbose",LongOpt.OPTIONAL_ARGUMENT,
			      argsb, 'v');
    longopts[2] = new LongOpt("nocompress",LongOpt.NO_ARGUMENT, null, 'n');
    longopts[3] = new LongOpt("version",LongOpt.NO_ARGUMENT, null, 'V');

    Getopt g = new Getopt(progname, args, "vnhV?", longopts);
    g.setOpterr(false); // We'll do our own error handling


    while ((c = g.getopt()) != -1)
      switch (c) {
      case 'n':
	// Flag -n or option --nocompress
	compress = 0;
	break;
      case 'h':
      case '?':
	usage();
	return;
      case 'v':
	// Flag -v, allow to increment the value
	verboseLevel += 1;
	break;
      case 'V':
	// Flag -V, show version info and terminate
	showVersion();
	break;
      case 0:
	// got long option.
	switch ((c=(new Integer(argsb.toString())).intValue())) {
	  // Find out by index, which should also be the last number
	  // given in the definition, which option we got.
	case 'v':
	  // Option --verbose, allow to set the value of verbosity
	  optval = g.getOptarg();
	  if (optval == null)
	    verboseLevel+=1;
	  else {
	    verboseLevel = Integer.valueOf(optval).intValue();
	  }
	  break;
	default:
	  msgout("Unknown long argument no. "+((char)c)+".");
	  break;
	}
	break;
      default:
	msgout("Unknown argument '"+((char)c)+"' ("+c+").");
	usage();
	return;
      }


    // Now check for needed arguments. If these are missing or if help
    // was requested, display usage message.
    if (g.getOptind()+2 != args.length) {
      msgout("File arguments are missing.");
      usage();
      return;
    }

    // Now start processing by setting up file names.

    String infile = args[g.getOptind()];
    String outfile = args[g.getOptind()+1];

    if (verboseLevel>0) {
      msgout("This is "+progname+" version "+version+".");
    }
    // create the processing engine
    PP4Pages pp4 = new PP4Pages(infile,outfile,System.out,
				progname+" V"+version);
    // set some options
    pp4.setCompressLevel(compress);
    pp4.setVerboseLevel(verboseLevel);

    // create the known handlers
    pp4.addCommentProcessor(new PP4hdPause());
    pp4.addCommentProcessor(new PP4hdEsuap());
    pp4.addCommentProcessor(new PP4hdMonoBg());
    pp4.addCommentProcessor(new PP4hdGradBg());

    pp4.work();
  }
}

/*
 * $Log: PPower4.java,v $
 * Revision 1.11  2000/05/04 07:17:39  gunterma
 * fix background color processing for grayscale values
 *
 * Revision 1.10  2000/05/03 09:49:20  gunterma
 * fix problem with empty lines in pdf stream objects
 *
 * Revision 1.9  2000/04/12 06:09:04  gunterma
 * Fix a problem with inserting the "first" markers.
 *
 * Revision 1.8  2000/04/10 06:35:39  gunterma
 * make this version compatible with Java 1.1.x
 *
 * Revision 1.7  2000/04/03 14:13:05  gunterma
 * Optional arguments processing fixed.
 * Added --version/-V flag to main program.
 * PPower4 version 0.8 to be released.
 *
 * Revision 1.6  2000/03/13 09:48:15  gunterma
 * Allow optional arguments when processing a pause to define a new
 * transition mode for the created page.
 *
 * Revision 1.5  1999/12/06 09:28:27  gunterma
 * Update version number because of change in floating point fixes to pj-library
 * for Kaffe 1.0.5
 *
 * Revision 1.4  1999/10/05 13:22:04  gunterma
 * version update due to fix of PjNumber in pj-0.22
 *
 * Revision 1.3  1999/09/07 05:47:45  gunterma
 * update license and warranty information
 *
 * Revision 1.2  1999/08/26 08:02:22  gunterma
 * Make sure that calculations for the background colors are not written
 * in localized conventions.
 *
 * Revision 1.1  1999/08/09 11:56:50  gunterma
 * initial checkin
 *
 */
