# $Id: Rules.mk,v 1.1 1999/08/09 11:53:18 gunterma Exp $
#----------------------------------------------------------------------
# Common Makefile definitions and rules for Java projects.
#
# [history at end]
#

#----------------------------------------------------------------------
# Basic setup

# first determine relative path to project top directory
ifdef ProjectTop
TopDir := .
else
# we are in a subdir, locate top directory by searching upward in tree
# for subdirectory 'project' which hopefully only exists at the top
# level
TopDir := $(shell \
  d='.'; \
  while [ ! -d "$${d}/project" ]; \
  do \
    d="$$d/.."; \
    done; \
  echo $$d)
endif

ProjectDir := $(TopDir)/project


#----------------------------------------------------------------------
# Project configuration

# project description
-include $(ProjectDir)/project.mk

# site description
-include $(ProjectDir)/site.mk


#----------------------------------------------------------------------
# Programs:

# JVM
Java := $(JDKDir)/bin/java

# Java Compiler
JavaC := $(JDKDir)/bin/javac

ifdef DebugInfo
JavaCFlags := -g
else
JavaCFlags := -O
endif

# JavaDoc generator
ifdef UseJavaDoc1
JavaDoc := $(JavaDoc1)
JavaDocFlags := -author -version $(JavaDocOptions)

else
JavaDoc := $(JavaDoc2)

JavaDocFlags := -author -version \
                -doctitle '$(ProjectTitle)' \
                -windowtitle '$(ProjectShortTitle) API'

ifdef JavaDocLinkURLs
JavaDocFlags += $(addprefix -link ,$(JavaDocLinkURLs))
endif

ifdef JavaDocOptions
JavaDocFlags += $(JavaDocOptions)
endif

endif

# JUnit regression tester
JUnitBatch := JAVA_VERSION=$(JavaVersion) junit -batch
JUnit := JAVA_VERSION=$(JavaVersion) junit

# Java package name handling
ifdef JavaPackageName
JavaPackagePrefix := $(JavaPackageName).
JavaPackageRelDir := $(subst .,/,$(JavaPackageName))
endif


#----------------------------------------------------------------------
# Classpath setup:

# now we can construct our classpath
ClassPath := $(TopDir)

ifdef ProjectClassPath
ClassPath := $(ClassPath):$(ProjectClassPath)
endif

ifdef SiteClassPath
ClassPath := $(ClassPath):$(SiteClassPath)
endif

# classpath without test classes
ReleaseClassPath := $(ClassPath)

# append paths to JUnit and project test cases, if requested
ifdef JUnitTests
ClassPath := $(ClassPath):$(TopDir)/junit:$(JUnitDir)/junit.jar
endif


#----------------------------------------------------------------------
# Rules


# default action is just to compile everything
default: all


# toplevel Makefile
ifdef ProjectTop

# things to do here:
world: setup all doc tests

# determine directories to make
_JavaPackageSubdirs := $(subst .,/,$(JavaPackages))
SubDirs += $(_JavaPackageSubdirs)

# generate API documentation
doc::
	mkdir -p api
	CLASSPATH=$(ReleaseClassPath) $(JavaDoc) -d api \
	  $(JavaDocFlags) $(JavaPackages)

realclean::
	-rm -rf api


# regression test rules
ifdef JUnitTests
SubDirs += $(addprefix junit/,$(_JavaPackageSubdirs)) junit

tests::
	CLASSPATH=$(ClassPath) $(JUnitBatch) AllTests

gui-tests::
	CLASSPATH=$(ClassPath) $(JUnit) AllTests &
endif

endif


# Subsystem rules
#
# If subdirectories are present process them recursively.
ifdef SubDirs

# handle subdir only if it has a Makefile too
define makeSubdirs
$(foreach dir,$(SubDirs),\
  test -r $(dir)/Makefile && $(MAKE) -C $(dir) $@;)
endef

all:: ; $(makeSubdirs)
setup:: ; $(makeSubdirs)
clean:: ; $(makeSubdirs)
realclean:: ; $(makeSubdirs)

endif


# Java classes
ifdef JavaSources
_JavaClassFiles := $(JavaSources:.java=.class)

# make sure all changed sources will be compiled at once
all:: .compiled

.compiled: $(JavaSources)
	CLASSPATH=$(ClassPath) $(JavaC) $(JavaCFlags) $?
	@touch .compiled

# this rule is just in case a single class file is requested
%.class: %.java
	CLASSPATH=$(ClassPath) $(JavaC) $(JavaCFlags) $<

FilesToClean += .compiled $(_JavaClassFiles)
endif


# Cleanup
ifdef FilesToClean
clean::
	-rm -f $(FilesToClean)
endif

ifdef FilesToRealClean
realclean::
	-rm -f $(FilesToRealClean)
endif

realclean:: clean


# hints
.PHONY: all setup clean realclean doc tests gui-tests

#----------------------------------------------------------------------
#
# $Log: Rules.mk,v $
# Revision 1.1  1999/08/09 11:53:18  gunterma
# initial checkin
#
#
